#
# inst.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#

require 'rbconfig'
require 'ftools'
require 'tempfile'

require 'amstd/futils'


module Installer

  include FileUtils
  include Config

  def Installer.sepjoin( *args )
    args.join File::Separator
  end

  BINDIR    = sepjoin( CONFIG['bindir'] )
  LIBDIR    = sepjoin( CONFIG['libdir'],
                       CONFIG['ruby_install_name'],
                       CONFIG['MAJOR'] + '.' + CONFIG['MINOR'],
                       'site_ruby' )
  RBDIR     = sepjoin( LIBDIR )
  SODIR     = sepjoin( LIBDIR, CONFIG['arch'] )

  RUBY_PATH = sepjoin( CONFIG['bindir'],
                       CONFIG['ruby_install_name'] )

  class << self

    def execute
      if not ARGV[0] then
        try_setup;   puts 'setup ok, entering installation step ...'
        try_install; puts 'done installation.'

      elsif ARGV[0] == 'setup'   then try_setup
      elsif ARGV[0] == 'install' then try_install
      else
        puts 'usage: ruby setup.rb [setup|install]'
        exit 1
      end
    end

    def try_setup
      begin
        com_setup
      rescue
        puts 'setup failed'
        puts $!
        exit 1
      end
    end

    def try_install
      begin
        com_install
      rescue
        puts 'install failed'
        puts $!
        exit 1
      end
    end


    def install_library( libn )
      lib_install( libn ) do |rb_to, so_to|
        install_rb rb_to
        each_dir do |ext|
          make_install_so so_to
        end
      end
    end

    def lib_install( libname )
      chdir( libname ) do
        yield newdir( RBDIR + '/' + libname, SODIR + '/' + libname )
      end
    end


    def install_rb( to )
      Dir.foreach( '.' ) do |fn|
        if /\.rb\z/o === fn and file? fn then
          install fn, to, 644, true
        end
      end
    end


    def extconf
      load './extconf.rb' unless is_newer? 'Makefile', 'extconf.rb'
    end

    def make
      system 'make'
    end

    def compile_all_in( dir )
      chdir( dir ) do
        each_dir do |ext|
          extconf
          make
        end
      end
    end

    def make_install( to )
      find_so('.').each do |fn|
        install fn, to, 555, true
      end
    end

    def find_so( dir = '.' )
      fnames = nil
      Dir.open( dir ) {|d| fnames = d.to_a }
      ['so', 'sl', 'dll', 'o'].each do |ext|
        exp = /\.#{ext}\z/
        a = fnames.find_all {|f| exp === f }
        return a if Array === a
      end
      
      raise ArgumentError,
        "can't find ruby extention library in '#{expand_path dir}'"
    end

    def each_dir
      Dir.foreach( '.' ) do |fn|
        next if /\A\./o === fn
        if directory? fn then
          chdir( fn ) do
            yield fn
          end
        end
      end
    end


    def install_all( to )
      Dir.foreach( '.' ) do |fn|
        if file? fn then
          install fn, to, 644, true
        end
      end
    end


    def add_rubypath( fn, opt = '' )
      opt = ' ' + opt unless opt.empty?

      tmpf = Tempfile.open( 'rbinst' )
      File.open( fn ) do |f|
        first = f.gets
        tmpf.puts "\#!#{RUBY_PATH}#{opt}"
        tmpf << first unless /\A\#\!/o === first
        f.each {|i| tmpf << i }
      end
      tmpf.close
      
      mod = stat( fn ).mode
      tmpf.open
      open( fn, 'w' ) do |wf|
        tmpf.each {|i| wf << i }
      end
      chmod mod, fn

      tmpf.close true
    end

  end

end
