#
# packing.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#

require 'amstd/common'
require 'amstd/futils'


module Packing


  class Scope

    include FileUtils

    def initialize( name, stack )
      @name = name.dup
      @stack = stack
      @values = {}

      @verbose = false
    end

    attr :name

    def []( name )
      @values[ _name2str name ]
    end

    def []=( name, val )
      @values[ _name2str name ] = val
    end


    def set( name, val )
      if String === name and (a = name.split('.')).size > 1 then
        @stack[ a[0] ][ a[1] ] = val
      else
        self[ name ] = val
      end
    end

    alias s set

    def get( name )
      if String === name and (a = name.split('.')).size > 1 then
        ret = @stack[ a[0] ][ a[1] ]
        warn "#{name} is nil" unless ret
      else
        ret = self[ name ]
        warn "#{self.name}.#{_name2str(name)} is nil" unless ret
      end

      ret
    end

    alias g get

    def warn( mes )
      $stderr.print 'Warning: '
      $stderr.puts mes
    end


    def get_env( name )
      @stack[ name ]
    end

    alias e get_env


    def a( arg )     # to Absolute path
      d( (g :dir), arg )
    end

    def d( dir, arg )    # in directory
      if Array === arg then
        arg.collect{|f| dir + '/' + f }
      else
        "#{dir}/#{arg}"
      end
    end

  end


  class ScopeError < StandardError ; end

  class ScopeStack

    def initialize
      @vals = {}
      @current = nil
      @prev = nil
      @stack = []
    end


    attr :current

    def []( name )
      @vals[ _name2str( name ) ]
    end

    def enter( name )
      name = _name2str( name )
      if s = @vals[ name ] then
        @current = s
      else
        @vals[ name ] = @current = Scope.new( name, self )
      end
      @stack.push @current

      @current
    end

    def leave( name = nil )
      name = _name2str( name ) if name
      unless s = @stack.pop then
        raise ScopeError, "too many leave than enter"
      end
      if name and s.name != name then
        puts "warning: leave from #{s.name}, but #{s.name} is required"
      end
      @prev = @current
      @current = @stack[-1]

      s
    end

    def invoke( task )
      @prev.send task
    end

  end


  MAIN_STACK = ScopeStack.new

  def environ( envname, &block )
    cur = MAIN_STACK.enter( envname )
    cur.instance_eval &block
    MAIN_STACK.leave envname
  end

  def invoke( task )
    MAIN_STACK.invoke task
  end

end


include Packing
